import unittest

from textractor.entities.line import Line
from textractor.entities.word import Word
from textractor.data.constants import TextTypes
from textractor.entities.bbox import BoundingBox
from textractor.visualizers.entitylist import EntityList

class TestLine(unittest.TestCase):
    def setUp(self):
        self.word_bb_1 = {
            "Width": 0.10809839516878128,
            "Height": 0.01363567914813757,
            "Left": 0.036161474883556366,
            "Top": 0.03439946100115776,
        }
        self.word_bb_2 = {
            "Width": 0.18033172190189362,
            "Height": 0.01742148958146572,
            "Left": 0.22032427787780762,
            "Top": 0.03645794093608856,
        }
        self.word_bb_3 = {
            "Width": 0.03744738921523094,
            "Height": 0.016524378210306168,
            "Left": 0.4087739884853363,
            "Top": 0.0368686243891716,
        }
        self.line_bb = {
            "Width": 0.3,
            "Height": 0.01742148958146572,
            "Left": 0.036161474883556366,
            "Top": 0.03439946100115776,
        }

        self.word_1 = Word(
            entity_id="word-id-1",
            bbox=BoundingBox.from_normalized_dict(self.word_bb_1, spatial_object=None),
            text="TEST",
            text_type=TextTypes.PRINTED,
        )
        self.word_2 = Word(
            entity_id="word-id-2",
            bbox=BoundingBox.from_normalized_dict(self.word_bb_2, spatial_object=None),
            text="WORDS",
            text_type=TextTypes.HANDWRITING,
        )
        self.word_3 = Word(
            entity_id="word-id-3",
            bbox=BoundingBox.from_normalized_dict(self.word_bb_3, spatial_object=None),
            text="ADDED",
            text_type=TextTypes.PRINTED,
        )

        self.line = Line(
            "line-id",
            BoundingBox.from_normalized_dict(self.line_bb, spatial_object=None),
            [self.word_1, self.word_2, self.word_3],
        )


    def test_get_words_by_type(self):
        """Test case to filter words of the Line by their type"""
        self.assertEqual(self.line.get_words_by_type(TextTypes.PRINTED), EntityList([self.word_1, self.word_3]))
        self.assertEqual(self.line.get_words_by_type(TextTypes.HANDWRITING), EntityList([self.word_2]))


    def test_get_text(self):
        """Test case setter for the text attribute"""
        self.assertEqual(self.line.text, "TEST WORDS ADDED")


    def test_set_page(self):
        """Test case setter for the page attribute"""
        self.line.page = 2
        self.assertEqual(self.line.page, 2)


    def test_set_page_id(self):
        """Test case setter for the page_id attribute"""
        self.line.page_id = "page-id"
        self.assertEqual(self.line.page_id, "page-id")


    def test_repr(self):
        """Test case setter for the repr function"""
        self.assertEqual(self.line.__repr__(), "TEST WORDS ADDED")



import json
import os
import PIL
import unittest
from tests.utils import get_fixture_path
from textractor import Textractor
from textractor.entities.document import Document
from textractor.data.constants import TextractFeatures, AnalyzeIDFields
from textractor.exceptions import InvalidProfileNameError, NoImageException, S3FilePathMissing

from .utils import save_document_to_fixture_path

class TestTextractorAnalyzeID(unittest.TestCase):
    def setUp(self):
        # insert credentials and filepaths here to run test
        self.profile_name = "default"
        self.current_directory = os.path.abspath(os.path.dirname(__file__))
        self.image_path = os.path.join(self.current_directory, "fixtures/fake_id.png")
        self.image = PIL.Image.open(os.path.join(self.current_directory, "fixtures/fake_id.png"))

        if self.profile_name is None:
            raise InvalidProfileNameError(
                "Textractor could not be initialized. Populate profile_name with a valid input in tests/test_textractor.py."
            )
        if os.environ.get("CALL_TEXTRACT"):
            self.extractor = Textractor(
                profile_name=self.profile_name, kms_key_id=""
            )

    def test_analyze_id_from_path(self):
        # Testing local single image input
        if os.environ.get("CALL_TEXTRACT"):
            document = self.extractor.analyze_id(
                file_source=self.image_path,
            )
            with open(get_fixture_path(), "w") as f:
                json.dump(document.response, f)
        else:
            document = Document.open(get_fixture_path())

        self.assertIsInstance(document, Document)
        self.assertEqual(len(document.identity_documents), 1)
        self.assertEqual(len(document.identity_documents[0].fields), 21)
        self.assertEqual(document.identity_documents[0].get(AnalyzeIDFields.FIRST_NAME), "GARCIA")
        self.assertEqual(document.identity_documents[0][AnalyzeIDFields.FIRST_NAME], "GARCIA")
    
    def test_analyze_id_from_image(self):
        # Testing local single image input
        if os.environ.get("CALL_TEXTRACT"):
            document = self.extractor.analyze_id(
                file_source=self.image,
            )
            with open(get_fixture_path(), "w") as f:
                json.dump(document.response, f)
        else:
            document = Document.open(get_fixture_path())

        self.assertIsInstance(document, Document)
        self.assertEqual(len(document.identity_documents), 1)
        self.assertEqual(len(document.identity_documents[0].fields), 21)
        self.assertEqual(document.identity_documents[0].get("FIRST_NAME"), "GARCIA")
        self.assertEqual(document.identity_documents[0]["FIRST_NAME"], "GARCIA")

if __name__ == "__main__":
    test = TestTextractorAnalyzeID()
    test.setUp()
    test.test_analyze_id_from_path()


"""Tests for all Word class methods."""

import unittest

from textractor.entities.bbox import BoundingBox
from textractor.data.constants import TextTypes
from textractor.entities.word import Word

class TestWord(unittest.TestCase):
    def setUp(self):
        self.bounding_box = {
            "Width": 0.10809839516878128,
            "Height": 0.01363567914813757,
            "Left": 0.036161474883556366,
            "Top": 0.03439946100115776,
        }


    def test_set_text(self):
        """Test case setter for the text attribute"""
        word = Word(
            entity_id="word-id",
            bbox=BoundingBox.from_normalized_dict(self.bounding_box, spatial_object=None),
        )
        word.text = "word-test"
        self.assertEqual(word.text, "word-test")


    def test_set_text_type(self):
        """Test case setter for the text type attribute"""
        word = Word(
            entity_id="word-id",
            bbox=BoundingBox.from_normalized_dict(self.bounding_box, spatial_object=None),
        )
        word.text_type = TextTypes.HANDWRITING
        self.assertEqual(word.text_type, TextTypes.HANDWRITING)


    def test_set_page(self):
        """Test case setter for the page attribute"""
        word = Word(
            entity_id="word-id",
            bbox=BoundingBox.from_normalized_dict(self.bounding_box, spatial_object=None),
        )
        word.page = 2
        self.assertEqual(word.page, 2)


    def test_set_page_id(self):
        """Test case setter for the page_id attribute"""
        word = Word(
            entity_id="word-id",
            bbox=BoundingBox.from_normalized_dict(self.bounding_box, spatial_object=None),
        )
        word.page_id = "page-id"
        self.assertEqual(word.page_id, "page-id")


    def test_repr(self):
        """Test case setter for the repr function"""
        word = Word(
            entity_id="word-id",
            bbox=BoundingBox.from_normalized_dict(self.bounding_box, spatial_object=None),
        )
        word.text = "word-test"
        self.assertEqual(word.__repr__(), "word-test")



import json
import os
import unittest
import PIL
from tests.utils import get_fixture_path
from textractor import Textractor
from textractor.entities.document import Document
from textractor.entities.word import Word
from textractor.entities.line import Line
from textractor.entities.page import Page
from textractor.entities.table import Table
from textractor.entities.value import Value
from textractor.data.constants import TableFormat
from textractor.entities.key_value import KeyValue
from textractor.visualizers.entitylist import EntityList
from textractor.exceptions import InvalidProfileNameError
from textractor.entities.selection_element import SelectionElement
from textractor.data.constants import TextTypes, SimilarityMetric, TextractFeatures, Direction, DirectionalFinderType

from .utils import save_document_to_fixture_path

try:
    import sentence_transformers
    SENTENCE_TRANSFORMERS_AVAILABLE = True
except:
    SENTENCE_TRANSFORMERS_AVAILABLE = False 

class TestPage(unittest.TestCase):
    def test_page(self):
        profile_name = "default"
        current_directory = os.path.abspath(os.path.dirname(__file__))

        if profile_name is None:
            raise InvalidProfileNameError(
                "Textractor could not be initialized. Populate profile_name with a valid input in tests/test_table.py."
            )

        if os.environ.get("CALL_TEXTRACT"):
            extractor = Textractor(profile_name=profile_name, kms_key_id="")
            document = extractor.analyze_document(
                file_source=os.path.join(current_directory, "fixtures/single-page-1.png"),
                features=[TextractFeatures.TABLES, TextractFeatures.FORMS],
            )
            with open(get_fixture_path(), "w") as f:
                json.dump(document.response, f)
        else:
            document = Document.open(get_fixture_path())

        page = document.page(0)

        self.assertIsInstance(page.words, EntityList)
        self.assertIsInstance(page.words[0], Word)
        self.assertEqual(len(page.words), 51)

        self.assertIsInstance(page.lines, EntityList)
        self.assertIsInstance(page.lines[0], Line)
        self.assertEqual(len(page.lines), 24)

        self.assertIsInstance(page.key_values, EntityList)
        self.assertIsInstance(page.key_values[0], KeyValue)
        self.assertIsInstance(page.key_values[0].value, Value)
        self.assertEqual(len(page.key_values), 3)
        self.assertIsInstance(page.key_values.pretty_print(), str)
        self.assertGreater(len(page.tables.pretty_print(table_format=TableFormat.CSV)), 100)


        self.assertIsInstance(page.checkboxes, EntityList)
        self.assertIsInstance(page.checkboxes[0], KeyValue)
        self.assertIsInstance(page.checkboxes[0].value, Value)
        self.assertEqual(page.checkboxes[0].value.words, [])
        self.assertIsInstance(page.checkboxes[0].value.children[0], SelectionElement)
        self.assertEqual(len(page.checkboxes), 2)

        self.assertIsInstance(page.tables, EntityList)
        self.assertEqual(len(page.tables), 1)
        self.assertIsInstance(page.tables.pretty_print(), str)
        self.assertGreater(len(page.tables.pretty_print()), 100)

        self.assertEqual(
            page.__repr__(),
            "This Page (1) holds the following data:\nWords - 51\nLines - 24\nKey-values - 3\nCheckboxes - 2\nTables - 1\nQueries - 0\nSignatures - 0\nExpense documents - 0\nLayout elements - 11"
        )

        self.assertEqual(len(page.keys(include_checkboxes=True)), 5)
        self.assertEqual(len(page.keys(include_checkboxes=False)), 3)

        self.assertEqual(len(page.filter_checkboxes(selected=True, not_selected=True)), 2)
        self.assertEqual(len(page.filter_checkboxes(selected=True, not_selected=False)), 1)
        self.assertEqual(len(page.filter_checkboxes(selected=False, not_selected=True)), 1)
        self.assertEqual(len(page.filter_checkboxes(selected=False, not_selected=False)), 0)

        self.assertIsInstance(page.get_words_by_type(), EntityList)
        self.assertIsInstance(page.get_words_by_type()[0], Word)
        self.assertEqual(len(page.get_words_by_type(TextTypes.PRINTED)), 51)
        self.assertEqual(len(page.get_words_by_type(TextTypes.HANDWRITING)), 0)

        if SENTENCE_TRANSFORMERS_AVAILABLE:
            self.assertIsInstance(
                page.search_words(
                    keyword="Table",
                    top_k=5,
                    similarity_metric=SimilarityMetric.COSINE,
                    similarity_threshold=0.6,
                ),
                EntityList,
            )
            self.assertIsInstance(
                page.search_words(
                    keyword="Table",
                    top_k=5,
                    similarity_metric=SimilarityMetric.COSINE,
                    similarity_threshold=0.6,
                )[0],
                Word,
            )
            self.assertIsInstance(
                page.search_words(
                    keyword="Table",
                    top_k=5,
                    similarity_metric=SimilarityMetric.EUCLIDEAN,
                    similarity_threshold=0.6,
                )[0],
                Word,
            )
            self.assertEqual(
                len(
                    page.search_words(
                        keyword="Table",
                        top_k=5,
                        similarity_metric=SimilarityMetric.COSINE,
                        similarity_threshold=0.6,
                    )
                ),
                1
            )

            self.assertIsInstance(
                page.search_lines(
                    keyword="Textractor",
                    top_k=5,
                    similarity_metric=SimilarityMetric.COSINE,
                    similarity_threshold=0.6,
                ),
                EntityList,
            )
            self.assertIsInstance(
                page.search_lines(
                    keyword="Textractor",
                    top_k=5,
                    similarity_metric=SimilarityMetric.COSINE,
                    similarity_threshold=0.6,
                )[0],
                Line,
            )
            self.assertIsInstance(
                page.search_lines(
                    keyword="Textractor",
                    top_k=5,
                    similarity_metric=SimilarityMetric.EUCLIDEAN,
                    similarity_threshold=0.6,
                )[0],
                Line,
            )
            self.assertEqual(
                len(
                    page.search_lines(
                        keyword="Textractor",
                        top_k=5,
                        similarity_metric=SimilarityMetric.COSINE,
                        similarity_threshold=0.6,
                    )
                ),
                2
            )

            self.assertIsInstance(
                page.get(
                    key="date",
                    top_k_matches=5,
                    similarity_metric=SimilarityMetric.COSINE,
                    similarity_threshold=0.6,
                ),
                EntityList,
            )
            self.assertIsInstance(
                page.get(
                    key="date",
                    top_k_matches=5,
                    similarity_metric=SimilarityMetric.COSINE,
                    similarity_threshold=0.6,
                )[0],
                KeyValue,
            )
            self.assertIsInstance(
                page.get(
                    key="date",
                    top_k_matches=5,
                    similarity_metric=SimilarityMetric.EUCLIDEAN,
                    similarity_threshold=0.6,
                )[0],
                KeyValue,
            )
            self.assertEqual(
                len(
                    page.get(
                        key="date",
                        top_k_matches=5,
                        similarity_metric=SimilarityMetric.COSINE,
                        similarity_threshold=0.6,
                    )
                ),
                1
            )
            self.assertEqual(
                len(page.directional_finder(
                    word_1 = "key-values",
                    word_2 = "table 1",
                    prefix = "",
                    direction=Direction.BELOW,
                    entities=[DirectionalFinderType.KEY_VALUE_SET, DirectionalFinderType.SELECTION_ELEMENT],
                )),
                3
            )

            self.assertEqual(len(page.directional_finder(
                    word_1 = "key-values",
                    word_2 = "",
                    prefix = "",
                    direction=Direction.LEFT,
                    entities=[DirectionalFinderType.KEY_VALUE_SET, DirectionalFinderType.SELECTION_ELEMENT],
                )), 0)

            self.assertEqual(len(page.directional_finder(
                    word_1 = "key-values",
                    word_2 = "",
                    prefix = "",
                    direction=Direction.RIGHT,
                    entities=[DirectionalFinderType.KEY_VALUE_SET, DirectionalFinderType.SELECTION_ELEMENT],
                )), 3)

            self.assertEqual(len(page.directional_finder(
                    word_1 = "key-values",
                    word_2 = "",
                    prefix = "",
                    direction=Direction.ABOVE,
                    entities=[DirectionalFinderType.KEY_VALUE_SET, DirectionalFinderType.SELECTION_ELEMENT],
                )), 0)

        self.assertIsInstance(
            page.search_words(
                keyword="Table",
                top_k=5,
                similarity_metric=SimilarityMetric.LEVENSHTEIN,
                similarity_threshold=5,
            )[0],
            Word,
        )
        self.assertIsInstance(
            page.get(
                key="date",
                top_k_matches=5,
                similarity_metric=SimilarityMetric.LEVENSHTEIN,
                similarity_threshold=5,
            )[0],
            KeyValue,
        )
        self.assertIsInstance(
            page.search_lines(
                keyword="Textractor",
                top_k=5,
                similarity_metric=SimilarityMetric.LEVENSHTEIN,
                similarity_threshold=5,
            )[0],
            Line,
        )

        save_file_path = os.path.join(current_directory, "Key-Values.csv")
        page.export_kv_to_csv(include_kv=True, include_checkboxes=True, filepath=save_file_path)
        self.assertIn("Key-Values.csv", os.listdir(current_directory))
        os.remove(save_file_path)

        save_file_path = os.path.join(current_directory, "Key-Values.txt")
        page.export_kv_to_txt(include_kv=True, include_checkboxes=True, filepath=save_file_path)
        self.assertIn("Key-Values.txt", os.listdir(current_directory))
        os.remove(save_file_path)

        save_file_path = os.path.join(current_directory, "Tables.xlsx")
        page.export_tables_to_excel(filepath=save_file_path)
        self.assertIn("Tables.xlsx", os.listdir(current_directory))
        os.remove(save_file_path)

        self.assertEqual(len(page.independent_words()), 10)
        self.assertIsInstance(page.independent_words()[0], Word)

        self.assertIsInstance(page.independent_words(), EntityList)
        self.assertIsInstance(page.independent_words()[0], Word)

        self.assertIsInstance(page.return_duplicates(), list)
        self.assertIsInstance(page.return_duplicates()[0], EntityList)



import os
import unittest
from tests.utils import get_fixture_path
from textractor import Textractor
from textractor.entities.document import Document
from textractor.exceptions import InputError, InvalidProfileNameError
from textractor.data.constants import TextractFeatures

from .utils import save_document_to_fixture_path

class QueriesTests(unittest.TestCase):
    def test_queries_as_strings(self):
        profile_name = "default"
        current_directory = os.path.abspath(os.path.dirname(__file__))

        if profile_name is None:
            raise InvalidProfileNameError(
                "Textractor could not be initialized. Populate profile_name with a valid input in tests/test_table.py."
            )

        if os.environ.get("CALL_TEXTRACT"):
            extractor = Textractor(profile_name=profile_name, kms_key_id="")
            document = extractor.analyze_document(
                file_source=os.path.join(current_directory, "fixtures/single-page-1.png"),
                features=[TextractFeatures.QUERIES],
                queries=[
                    "What is the name of the package?",
                    "What is the title of the document?",
                ],
            )
        else:
            document = Document.open(get_fixture_path())

        self.assertEqual(len(document.queries), 2)
        self.assertEqual(document.queries[0].result.answer, "Textractor")
        self.assertEqual(document.queries[1].result.answer, "Textractor Test Document")

    def test_bad_queries_as_strings(self):
        profile_name = "default"
        current_directory = os.path.abspath(os.path.dirname(__file__))

        if profile_name is None:
            raise InvalidProfileNameError(
                "Textractor could not be initialized. Populate profile_name with a valid input in tests/test_table.py."
            )

        if os.environ.get("CALL_TEXTRACT"):
            extractor = Textractor(profile_name=profile_name, kms_key_id="")
            document = extractor.analyze_document(
                file_source=os.path.join(current_directory, "fixtures/single-page-1.png"),
                features=[TextractFeatures.QUERIES],
                queries=[
                    "Lorem ipsum?",
                    "The quick brown fox jumps over the lazy dog?",
                ],
            )
        else:
            document = Document.open(get_fixture_path())

        self.assertEqual(len(document.queries), 2)
        self.assertEqual(document.queries[0].result, None)
        self.assertEqual(document.queries[1].result, None)

    @unittest.skipIf(not os.environ.get("CALL_TEXTRACT"), "Asynchronous requests can't be processed without calling Textract")
    def test_query_feature_without_queries(self):
        profile_name = "default"
        current_directory = os.path.abspath(os.path.dirname(__file__))

        if profile_name is None:
            raise InvalidProfileNameError(
                "Textractor could not be initialized. Populate profile_name with a valid input in tests/test_table.py."
            )

        extractor = Textractor(profile_name=profile_name, kms_key_id="")
        with self.assertRaises(InputError):
            document = extractor.analyze_document(
                file_source=os.path.join(current_directory, "fixtures/single-page-1.png"),
                features=[TextractFeatures.TABLES],
                queries=[
                    "Lorem ipsum?",
                    "The quick brown fox jumps over the lazy dog?",
                ],
            )



import unittest

from textractor.data.constants import TextTypes
from textractor.entities.value import Value
from textractor.entities.word import Word
from textractor.entities.bbox import BoundingBox
from textractor.visualizers.entitylist import EntityList

class TestValue(unittest.TestCase):
    def setUp(self):
        self.word_bb_1 = {
            "Width": 0.10809839516878128,
            "Height": 0.01363567914813757,
            "Left": 0.036161474883556366,
            "Top": 0.03439946100115776,
        }
        self.word_bb_2 = {
            "Width": 0.18033172190189362,
            "Height": 0.01742148958146572,
            "Left": 0.22032427787780762,
            "Top": 0.03645794093608856,
        }
        self.word_bb_3 = {
            "Width": 0.03744738921523094,
            "Height": 0.016524378210306168,
            "Left": 0.4087739884853363,
            "Top": 0.0368686243891716,
        }
        self.value_bb = {
            "Width": 0.02524515800178051,
            "Height": 0.01746263913810253,
            "Left": 0.18779051303863525,
            "Top": 0.4229613244533539,
        }

        self.word_1 = Word(
            entity_id="word-id-1",
            bbox=BoundingBox.from_normalized_dict(self.word_bb_1, spatial_object=None),
            text="TEST",
            text_type=TextTypes.PRINTED,
        )
        self.word_2 = Word(
            entity_id="word-id-2",
            bbox=BoundingBox.from_normalized_dict(self.word_bb_2, spatial_object=None),
            text="WORDS",
            text_type=TextTypes.HANDWRITING,
        )
        self.word_3 = Word(
            entity_id="word-id-3",
            bbox=BoundingBox.from_normalized_dict(self.word_bb_3, spatial_object=None),
            text="ADDED",
            text_type=TextTypes.PRINTED,
        )

        self.value = Value(
            entity_id="value-id",
            bbox=BoundingBox.from_normalized_dict(self.value_bb, spatial_object=None),
        )
        self.word_objs = [self.word_1, self.word_2, self.word_3]
        self.value.words = self.word_objs
        self.value.key_id = "key-id"
        self.value.contains_checkbox = False
        self.value.page = 2
        self.value.page_id = "page-id"


    def test_words(self):
        """Test case to add words to the Value field of a key-value pair"""
        self.assertEqual(self.value.words, EntityList(self.word_objs))
    
    
    def test_key_id(self):
        """Test case to access Key ID of a key-value pair"""
        self.assertEqual(self.value.key_id, "key-id")
    
    
    def test_contains_checkbox(self):
        self.assertFalse(self.value.contains_checkbox)
    

    def test_set_page(self):
        """Test case setter for the page attribute"""
        self.assertEqual(self.value.page, 2)
    
    
    def test_set_page_id(self):
        """Test case setter for the page_id attribute"""
        self.assertEqual(self.value.page_id, "page-id")
    
    
    def test_get_words_by_type(self):
        """Test case to retrieve words of a specific type in the Value field of a key-value pair"""
        self.assertEqual(
            self.value.get_words_by_type(text_type=TextTypes.PRINTED),
            EntityList([self.word_1, self.word_3])
        )
        self.assertEqual(
            self.value.get_words_by_type(text_type=TextTypes.HANDWRITING),
            EntityList([self.word_2])
        )


    def test_repr(self):
        """Test case to retrieve words of the Value field in a key-value pair as text"""
        self.assertEqual(self.value.__repr__(), "TEST WORDS ADDED")



import os
import PIL
import unittest
import boto3
import uuid
import logging
from tests.utils import get_fixture_path

from textractor import Textractor
from textractor.entities.document import Document
from textractor.entities.lazy_document import LazyDocument
from textractor.data.constants import TextractFeatures
from textractor.exceptions import InvalidProfileNameError, S3FilePathMissing
from textractor.utils.s3_utils import upload_to_s3, delete_from_s3


class TestTextractor(unittest.TestCase):
    def setUp(self):
        # insert credentials and filepaths here to run test
        #self.profile_name = "default"
        self.bucket_name = os.environ.get("S3_BUCKET", "textractor-tests")
        if os.environ.get("CALL_TEXTRACT"):
            self.s3_client = boto3.session.Session(
                #profile_name=self.profile_name
                region_name="us-west-2"
            ).client("s3", region_name="us-west-2")

            self.current_directory = os.path.abspath(os.path.dirname(__file__))
            for asset in ["single-page-1.png", "textractor-multipage-doc.pdf"]:
                upload_to_s3(self.s3_client, f"s3://{self.bucket_name}/{asset}", os.path.join(self.current_directory, f"fixtures/{asset}"))

            self.s3_image_file = f"s3://{self.bucket_name}/single-page-1.png"
            self.s3_multipage_pdf_file = f"s3://{self.bucket_name}/textractor-multipage-doc.pdf"
            self.s3_output_path = f"s3://{self.bucket_name}/output"
            self.s3_upload_path = f"s3://{self.bucket_name}/upload"
            self.image_1 = PIL.Image.open(os.path.join(self.current_directory, "fixtures/single-page-1.png"))
            self.image_2 = PIL.Image.open(os.path.join(self.current_directory, "fixtures/single-page-2.png"))

            #if self.profile_name is None:
            #    raise InvalidProfileNameError(
            #        "Textractor could not be initialized. Populate profile_name with a valid input in tests/test_textractor.py."
            #    )
            self.extractor = Textractor(
                #profile_name=self.profile_name, kms_key_id=""
                region_name="us-west-2"
            )

    def test_detect_document_text(self):
        # Testing local single image input
        if os.environ.get("CALL_TEXTRACT"):
            document = self.extractor.detect_document_text(
                file_source=os.path.join(self.current_directory, "fixtures/single-page-1.png"),
            )
        else:
            document = Document.open(get_fixture_path())

        self.assertIsInstance(document, Document)
        self.assertEqual(len(document.pages), 1)

    def test_detect_document_text_single_page_pdf_input(self):
        # Testing single page pdf input
        if os.environ.get("CALL_TEXTRACT"):
            document = self.extractor.detect_document_text(
                file_source=os.path.join(self.current_directory, "fixtures/textractor-singlepage-doc.pdf"),
                save_image=False,
            )
        else:
            document = Document.open(get_fixture_path())

        self.assertIsInstance(document, Document)
        self.assertIs(document.pages[0].image, None)

    def test_detect_document_text_list_PIL_images(self):
        # Testing list of PIL image input
        if os.environ.get("CALL_TEXTRACT"):
            document = self.extractor.detect_document_text(
                file_source=[self.image_1],
            )
        else:
            document = Document.open(get_fixture_path())

        self.assertIsInstance(document, Document)
        self.assertEqual(len(document.pages), 1)

    @unittest.skipIf(not os.environ.get("CALL_TEXTRACT"), "Asynchronous requests can't be processed without calling Textract")
    def test_textractor_pil_image_input(self):
        # Test PIL image input
        image_1 = PIL.Image.open(os.path.join(self.current_directory, "fixtures/single-page-1.png"))
        document = self.extractor.detect_document_text(
            file_source=image_1,
        )

    def test_textractor_s3_image_input(self):
        # Test S3 location path input
        if os.environ.get("CALL_TEXTRACT"):
            if self.s3_image_file is None:
                raise S3FilePathMissing(
                    "S3 URI needed to run test. Populate s3_image_file with a valid input in tests/test_textractor.py."
                )
            document = self.extractor.detect_document_text(
                file_source=self.s3_image_file,
            )
        else:
            document = Document.open(get_fixture_path())

        self.assertIsInstance(document, Document)

    @unittest.skipIf(not os.environ.get("CALL_TEXTRACT"), "Asynchronous requests can't be processed without calling Textract")
    def test_textractor_start_document_text_detection(self):
        # Testing start_document_text_detection() with local multipage pdf input
        document = self.extractor.start_document_text_detection(
            file_source=os.path.join(self.current_directory, "fixtures/textractor-multipage-doc.pdf"),
            s3_output_path=self.s3_output_path,
            s3_upload_path=self.s3_upload_path,
        )

        self.assertEqual(len(document.pages), 2)
        self.assertIsInstance(document, LazyDocument)

    def test_textractor_analyze_document(self):
        # Testing analyze_document() with local single image input
        if os.environ.get("CALL_TEXTRACT"):
            document = self.extractor.analyze_document(
                file_source=os.path.join(self.current_directory, "fixtures/single-page-1.png"),
                features=[TextractFeatures.TABLES, TextractFeatures.FORMS],
            )
        else:
            document = Document.open(get_fixture_path())

        self.assertEqual(len(document.pages), 1)
        self.assertIsInstance(document, Document)

    def test_textractor_analyze_document_local_pillow_image(self):
        # Testing analyze_document() with local PIL image input
        if os.environ.get("CALL_TEXTRACT"):
            document = self.extractor.analyze_document(
                file_source=self.image_1,
                features=[TextractFeatures.TABLES, TextractFeatures.FORMS],
            )
        else:
            document = Document.open(get_fixture_path())
        
        self.assertEqual(len(document.pages), 1)
        self.assertIsInstance(document, Document)

    def test_textractor_analyze_document_pillow_image_list(self):
        # Testing analyze_document() with local single image input
        if os.environ.get("CALL_TEXTRACT"):
            document = self.extractor.analyze_document(
                file_source=[self.image_1],
                features=[TextractFeatures.TABLES, TextractFeatures.FORMS],
                save_image=True,
            )
        else:
            document = Document.open(get_fixture_path())
        
        self.assertEqual(len(document.pages), 1)
        self.assertIsInstance(document, Document)

    @unittest.skipIf(not os.environ.get("CALL_TEXTRACT"), "Asynchronous requests can't be processed without calling Textract")
    def test_textractor_analyze_document_multipage_pdf(self):
        # Testing start_document_analysis() with local multipage pdf input
        document = self.extractor.start_document_analysis(
            file_source=os.path.join(self.current_directory, "fixtures/textractor-multipage-doc.pdf"),
            features=[TextractFeatures.TABLES, TextractFeatures.FORMS],
            s3_output_path=self.s3_output_path,
            s3_upload_path=self.s3_upload_path,
        )
        
        self.assertIsInstance(document, LazyDocument)
        self.assertEqual(len(document.pages), 2)

    @unittest.skipIf(not os.environ.get("CALL_TEXTRACT"), "Asynchronous requests can't be processed without calling Textract")
    def test_textractor_start_document_text_detection_multipage_pdf_s3(self):
        # Testing start_document_text_detection() with s3 multipage pdf input
        document = self.extractor.start_document_text_detection(
            file_source=self.s3_multipage_pdf_file,
            s3_output_path=self.s3_output_path,
        )
        
        self.assertEqual(len(document.pages), 2)
        self.assertIsInstance(document, LazyDocument)

    @unittest.skipIf(not os.environ.get("CALL_TEXTRACT"), "Asynchronous requests can't be processed without calling Textract")
    def test_textractor_start_document_analysis_multipage_pdf_s3(self):
        # Testing start_document_analysis() with s3 multipage pdf input
        document = self.extractor.start_document_analysis(
            file_source=self.s3_multipage_pdf_file,
            features=[TextractFeatures.TABLES, TextractFeatures.FORMS],
            s3_output_path=self.s3_output_path,
        )
        
        self.assertIsInstance(document, LazyDocument)
        self.assertEqual(len(document.pages), 2)

    @unittest.skipIf(not os.environ.get("CALL_TEXTRACT"), "Asynchronous requests can't be processed without calling Textract")
    def test_textractor_start_document_analysis(self):
        # Testing start_document_analysis() with local PIL Image input
        document = self.extractor.start_document_analysis(
            file_source=self.image_1,
            features=[TextractFeatures.TABLES, TextractFeatures.FORMS],
            s3_output_path=self.s3_output_path,
            s3_upload_path=self.s3_upload_path,
        )
        
        self.assertIsInstance(document, LazyDocument)
        self.assertEqual(len(document.pages), 1)



import json
import os
import unittest
import PIL
from tests.utils import get_fixture_path
from textractor import Textractor
from textractor.entities.document import Document
from textractor.entities.word import Word
from textractor.entities.line import Line
from textractor.entities.page import Page
from textractor.entities.table import Table
from textractor.entities.value import Value
from textractor.data.constants import TableFormat
from textractor.entities.key_value import KeyValue
from textractor.visualizers.entitylist import EntityList
from textractor.exceptions import InvalidProfileNameError
from textractor.entities.selection_element import SelectionElement
from textractor.data.constants import TextTypes, SimilarityMetric, TextractFeatures, Direction, DirectionalFinderType

from .utils import save_document_to_fixture_path

class TestLayout(unittest.TestCase):
    def test_layout(self):
        profile_name = "default"
        current_directory = os.path.abspath(os.path.dirname(__file__))

        if profile_name is None:
            raise InvalidProfileNameError(
                "Textractor could not be initialized. Populate profile_name with a valid input in tests/test_table.py."
            )

        if os.environ.get("CALL_TEXTRACT"):
            extractor = Textractor(profile_name=profile_name, kms_key_id="")
            document = extractor.analyze_document(
                file_source=os.path.join(current_directory, "fixtures/paystub.jpg"),
                features=[TextractFeatures.LAYOUT, TextractFeatures.TABLES, TextractFeatures.FORMS],
            )
            with open(get_fixture_path(), "w") as f:
                json.dump(document.response, f)
        else:
            document = Document.open(get_fixture_path())

        print(document.text)



import unittest
import numpy
from textractor.entities.bbox import BoundingBox

class TestBoundingBox(unittest.TestCase):
    def test_bbox(self):
        dims = {"Width": 3, "Height": 4, "Left": 1, "Top": 2}
        bbox = BoundingBox.from_normalized_dict(dims, spatial_object=None)

        self.assertTrue(isinstance(bbox.as_denormalized_numpy(), numpy.ndarray))
        self.assertEqual(bbox.__repr__(), "x: 1, y: 2, width: 3, height: 4")



import json
import os
import unittest
from tests.utils import get_fixture_path
from textractor import Textractor
from textractor.entities.document import Document
from textractor.entities.word import Word
from textractor.entities.table import Table
from textractor.entities.table_cell import TableCell
from textractor.visualizers.entitylist import EntityList
from textractor.exceptions import InvalidProfileNameError
from textractor.data.constants import TextractFeatures, TextTypes, CellTypes

from .utils import save_document_to_fixture_path

try:
    import sentence_transformers
    SENTENCE_TRANSFORMERS_AVAILABLE = True
except:
    SENTENCE_TRANSFORMERS_AVAILABLE = False

class TestTable(unittest.TestCase):
    def test_table(self):
        # Insert credentials here to run test
        profile_name = "default"
        current_directory = os.path.abspath(os.path.dirname(__file__))

        if profile_name is None:
            raise InvalidProfileNameError(
                "Textractor could not be initialized. Populate profile_name with a valid input in tests/test_table.py."
            )

        if os.environ.get("CALL_TEXTRACT"):
            extractor = Textractor(
                profile_name=profile_name, kms_key_id=""
            )
            document = extractor.analyze_document(
                file_source=os.path.join(current_directory, "fixtures/single-page-1.png"),
                features=[TextractFeatures.TABLES],
                save_image=False,
            )
        else:
            document = Document.open(get_fixture_path())

        table = document.tables[0]

        self.assertIsInstance(table.words, EntityList)
        self.assertEqual(len(table.words), 28)
        self.assertIsInstance(table.words[0], Word)

        self.assertEqual(table.page, 1)
        self.assertIsInstance(table.page_id, str)

        self.assertEqual(len(table.get_words_by_type(TextTypes.PRINTED)), 28)
        self.assertEqual(len(table.get_words_by_type(TextTypes.HANDWRITING)), 0)

        self.assertEqual(table.get_table_range(), (3, 5))

        new_table = table.strip_headers()
        self.assertEqual(new_table.get_table_range(), (2, 5))

        self.assertEqual(len(table.get_cells_by_type(CellTypes.COLUMN_HEADER).keys()), 4)
        self.assertEqual(table[1:, 1:].get_table_range(), (2, 4))

        table.to_excel(os.path.join(current_directory, "table.xlsx"))
        self.assertIn("table.xlsx", os.listdir(current_directory))
        os.remove(os.path.join(current_directory, "table.xlsx"))

        if SENTENCE_TRANSFORMERS_AVAILABLE:
            self.assertEqual(table.get_columns_by_name(
                ["Cell 1"], similarity_threshold=0.9
            ).get_table_range(), (3, 1))

        cell = table.table_cells[0]
        self.assertEqual(cell.page, 1)
        self.assertIsInstance(cell.page_id, str)

        self.assertEqual(cell.row_index, 1)
        self.assertEqual(cell.col_index, 1)
        self.assertEqual(cell.row_span, 1)
        self.assertEqual(cell.col_span, 1)

        self.assertIsInstance(cell.words, EntityList)
        self.assertIsInstance(cell.words[0], Word)

        self.assertIsInstance(cell.table_id, str) or cell.table_id is None
        self.assertEqual(len(cell.get_words_by_type(TextTypes.PRINTED)), 2)
        self.assertEqual(len(cell.get_words_by_type(TextTypes.HANDWRITING)), 0)

        self.assertEqual(cell.merge_direction(), (None, "None"))
        self.assertEqual(cell.__repr__(), "<Cell: (1,1), Span: (1, 1), Column Header: True, MergedCell: False>  Cell 1")

    def test_table_with_title_and_footers(self):
        # Insert credentials here to run test
        profile_name = "default"
        current_directory = os.path.abspath(os.path.dirname(__file__))

        if profile_name is None:
            raise InvalidProfileNameError(
                "Textractor could not be initialized. Populate profile_name with a valid input in tests/test_table.py."
            )

        if os.environ.get("CALL_TEXTRACT"):
            extractor = Textractor(
                profile_name=profile_name, kms_key_id=""
            )
            document = extractor.analyze_document(
                file_source=os.path.join(current_directory, "fixtures/paystub.jpg"),
                features=[TextractFeatures.TABLES],
                save_image=False,
            )
        else:
            document = Document.open(get_fixture_path())

        self.assertEqual(len(document.tables), 7)
        self.assertNotEqual(document.tables[3].title, None)
        self.assertEqual(len(document.tables[4].footers), 1)

if __name__ == "__main__":
    test = TestTable()
    test.setUp()
    test.test_table()


import json
import os
import PIL
import unittest
import boto3
import uuid
import logging
from lxml import html
from tests.utils import get_fixture_path

from textractor import Textractor
from textractor.data.constants import TextractFeatures
from textractor.data.text_linearization_config import TextLinearizationConfig
from textractor.data.html_linearization_config import HTMLLinearizationConfig
from textractor.entities.document import Document
from textractor.exceptions import InvalidProfileNameError, S3FilePathMissing
from textractor.utils.s3_utils import upload_to_s3, delete_from_s3

class TestGetTextAndWords(unittest.TestCase):
    """The tests below are smoke tests and are used to assess how a change impacts the output of Textractor.

    :param unittest: _description_
    :type unittest: _type_
    """
    def setUp(self):
        # insert credentials and filepaths here to run test
        self.profile_name = "default"
        if os.environ.get("CALL_TEXTRACT"):
            self.s3_client = boto3.session.Session(
                profile_name=self.profile_name
            ).client("s3", region_name="us-west-2")

            self.current_directory = os.path.abspath(os.path.dirname(__file__))

            self.extractor = Textractor(
                profile_name=self.profile_name, kms_key_id=""
            )
            self.fixture_directory = os.path.join(self.current_directory, "fixtures")

    def test_detect_no_duplicate_words(self):
        for asset in ["amzn_q2.png", "fake_id.png", "form_1005.png", "form.png", "in-table-title.png", "matrix.png", "patient_intake_form_sample.png", "paystub_header.png", "paystub_single_table.png", "paystub_tables.png", "paystub.jpg", "reading_order.pdf", "receipt.jpg", "sample-invoice.pdf", "screenshot.png", "single-page-1.png", "single-page-2.png", "test.png", "textractor-singlepage-doc.pdf", "tutorial.pdf"]:
            # Testing that no asset causes the output to contain duplicate words
            if os.environ.get("CALL_TEXTRACT"):
                document = self.extractor.analyze_document(
                    os.path.join(self.fixture_directory, asset),
                    features=[
                        TextractFeatures.LAYOUT,
                        TextractFeatures.TABLES,
                        TextractFeatures.FORMS,
                        TextractFeatures.SIGNATURES
                    ]
                )
                with open(get_fixture_path()[:-5] + "_" + asset + ".json", "w") as f:
                    json.dump(document.response, f)
            else:
                document = Document.open(get_fixture_path()[:-5] + "_" + asset + ".json")

            _, words = document.get_text_and_words()

            word_ids = set()
            for word in words:
                self.assertNotIn(word.id, word_ids, f"Word {word} ({word.id}) exists twice in the output for asset {asset}")
                word_ids.add(word.id)

    def test_detect_no_missing_words(self):
        # Change the path since we don't want to re-run Textract for all files
        fixture_path = get_fixture_path().replace("test_detect_no_missing_words", "test_detect_no_duplicate_words")
        for asset in ["amzn_q2.png", "fake_id.png", "form_1005.png", "form.png", "in-table-title.png", "matrix.png", "patient_intake_form_sample.png", "paystub_header.png", "paystub_single_table.png", "paystub_tables.png", "paystub.jpg", "reading_order.pdf", "receipt.jpg", "sample-invoice.pdf", "screenshot.png", "single-page-1.png", "single-page-2.png", "test.png", "textractor-singlepage-doc.pdf", "tutorial.pdf"]:
            # Testing that no asset causes the output to contain duplicate words
            if os.environ.get("CALL_TEXTRACT"):
                document = self.extractor.analyze_document(
                    os.path.join(self.fixture_directory, asset),
                    features=[
                        TextractFeatures.LAYOUT,
                        TextractFeatures.TABLES,
                        TextractFeatures.FORMS,
                        TextractFeatures.SIGNATURES
                    ]
                )
                with open(fixture_path[:-5] + "_" + asset + ".json", "w") as f:
                    json.dump(document.response, f)
            else:
                document = Document.open(fixture_path[:-5] + "_" + asset + ".json")

            _, words = document.get_text_and_words()

            original_word_set = set([b["Id"] for b in document.response["Blocks"] if b["BlockType"] == "WORD"])
            word_ids = set([w.id for w in words])

            self.assertTrue(original_word_set.issubset(word_ids))

    def test_table_prefixes_and_suffixes_in_text(self):
        if os.environ.get("CALL_TEXTRACT"):
            document = self.extractor.analyze_document(
                os.path.join(self.current_directory, "fixtures/single-page-1.png"),
                features=[
                    TextractFeatures.LAYOUT,
                    TextractFeatures.TABLES,
                    TextractFeatures.FORMS,
                    TextractFeatures.SIGNATURES
                ]
            )
            with open(get_fixture_path(), "w") as f:
                json.dump(document.response, f)
        else:
            document = Document.open(get_fixture_path())

        config = TextLinearizationConfig(
            title_prefix = "<title>",  #: Prefix for title layout elements
            title_suffix = "</title>",  #: Suffix for title layout elements
            table_layout_prefix = "<table_layout>",  #: Prefix for table elements
            table_layout_suffix = "</table_layout>",  #: Suffix for table elements
            table_prefix = "<table>",
            table_suffix = "</table>",
            table_row_prefix = "<tr>",  #: Prefix for table row
            table_row_suffix = "</tr>",  #: Suffix for table row
            table_cell_prefix = "<td>",  #: Prefix for table cell
            table_cell_suffix = "</td>",  #: Suffix for table cell
            table_cell_header_prefix = "<th>",  #: Prefix for header cell
            table_cell_header_suffix = "</th>",  #: Suffix for header cell
            header_prefix = "<header>",  #: Prefix for header layout elements
            header_suffix = "</header>",  #: Suffix for header layout elements
            section_header_prefix = "<section_header>",  #: Prefix for section header layout elements
            section_header_suffix = "</section_header>",  #: Suffix for section header layout elements
            text_prefix = "<text>",  #: Prefix for text layout elements
            text_suffix = "</text>",  #: Suffix for text layout elements
            key_value_layout_prefix = "<kv_layout>",  #: Prefix for key_value layout elements (not for individual key-value elements)
            key_value_layout_suffix = "</kv_layout>",  #: Suffix for key_value layout elements (not for individual key-value elements)
            key_value_prefix = "<kv>",  #: Prefix for key-value elements
            key_value_suffix = "</kv>",  #: Suffix for key-value elements
            key_prefix = "<key>",  #: Prefix for key elements
            key_suffix = "</key>",  #: Suffix for key elements
            value_prefix = "<value>",  #: Prefix for value elements
            value_suffix = "</value>",  #: Suffix for value elements
            add_prefixes_and_suffixes_in_text=True,
            add_prefixes_and_suffixes_as_words=True,
        )

        text, _ = document.get_text_and_words(config)

        for token in [
            "<title>",
            "</title>",
            "<table>",
            "</table>",
            "<table_layout>",
            "</table_layout>",
            "<tr>",
            "</tr>",
            "<td>",
            "</td>",
            # Sample does not have header cells
            #"<th>",
            #"</th>",
            # Sample does not have header
            #"<header>",
            #"</header>",
            "<section_header>",
            "</section_header>",
            # Sample does not have header
            #"<kv_layout>",
            #"</kv_layout>",
            "<kv>",
            "</kv>",
            "<key>",
            "</key>",
            "<value>",
            "</value>",
        ]:
            self.assertTrue(token in text, f"{token} is not in text")

    def test_table_prefixes_and_suffixes_in_words(self):
        if os.environ.get("CALL_TEXTRACT"):
            document = self.extractor.analyze_document(
                os.path.join(self.current_directory, "fixtures/single-page-1.png"),
                features=[
                    TextractFeatures.LAYOUT,
                    TextractFeatures.TABLES,
                    TextractFeatures.FORMS,
                    TextractFeatures.SIGNATURES
                ]
            )
            with open(get_fixture_path(), "w") as f:
                json.dump(document.response, f)
        else:
            document = Document.open(get_fixture_path())

        config = TextLinearizationConfig(
            title_prefix = "<title>",  #: Prefix for title layout elements
            title_suffix = "</title>",  #: Suffix for title layout elements
            table_layout_prefix = "<table_layout>",  #: Prefix for table elements
            table_layout_suffix = "</table_layout>",  #: Suffix for table elements
            table_prefix = "<table>",
            table_suffix = "</table>",
            table_row_prefix = "<tr>",  #: Prefix for table row
            table_row_suffix = "</tr>",  #: Suffix for table row
            table_cell_prefix = "<td>",  #: Prefix for table cell
            table_cell_suffix = "</td>",  #: Suffix for table cell
            table_cell_header_prefix = "<th>",  #: Prefix for header cell
            table_cell_header_suffix = "</th>",  #: Suffix for header cell
            header_prefix = "<header>",  #: Prefix for header layout elements
            header_suffix = "</header>",  #: Suffix for header layout elements
            section_header_prefix = "<section_header>",  #: Prefix for section header layout elements
            section_header_suffix = "</section_header>",  #: Suffix for section header layout elements
            text_prefix = "<text>",  #: Prefix for text layout elements
            text_suffix = "</text>",  #: Suffix for text layout elements
            key_value_layout_prefix = "<kv_layout>",  #: Prefix for key_value layout elements (not for individual key-value elements)
            key_value_layout_suffix = "</kv_layout>",  #: Suffix for key_value layout elements (not for individual key-value elements)
            key_value_prefix = "<kv>",  #: Prefix for key-value elements
            key_value_suffix = "</kv>",  #: Suffix for key-value elements
            key_prefix = "<key>",  #: Prefix for key elements
            key_suffix = "</key>",  #: Suffix for key elements
            value_prefix = "<value>",  #: Prefix for value elements
            value_suffix = "</value>",  #: Suffix for value elements
            add_prefixes_and_suffixes_in_text=True,
            add_prefixes_and_suffixes_as_words=True,
        )

        _, words = document.get_text_and_words(config)

        words = [w.text for w in words]

        for token in [
            "<title>",
            "</title>",
            "<table>",
            "</table>",
            "<table_layout>",
            "</table_layout>",
            "<tr>",
            "</tr>",
            "<td>",
            "</td>",
            # Sample does not have header cells
            #"<th>",
            #"</th>",
            # Sample does not have header
            #"<header>",
            #"</header>",
            "<section_header>",
            "</section_header>",
            # Sample does not have header
            #"<kv_layout>",
            #"</kv_layout>",
            "<kv>",
            "</kv>",
            "<key>",
            "</key>",
            "<value>",
            "</value>",
        ]:
            self.assertTrue(token in words, f"{token} is not in text")

    def test_figure_layout_prefixes_and_suffixes_in_text_words(self):
        if os.environ.get("CALL_TEXTRACT"):
            document = self.extractor.analyze_document(
                os.path.join(self.current_directory, "fixtures/matrix.png"),
                features=[
                    TextractFeatures.LAYOUT,
                ]
            )
            with open(get_fixture_path(), "w") as f:
                json.dump(document.response, f)
        else:
            document = Document.open(get_fixture_path())

        config = TextLinearizationConfig(
            figure_layout_prefix = "<figure>",  #: Prefix for figure elements
            figure_layout_suffix = "</figure>",  #: Suffix for figure elements
            add_prefixes_and_suffixes_in_text=True,
            add_prefixes_and_suffixes_as_words=True,
        )

        text, words = document.get_text_and_words(config)

        for token in [
            "<figure>",
            "</figure>",
        ]:
            self.assertTrue(token in text, f"{token} is not in text")

        words = [w.text for w in words]

        for token in [
            "<figure>",
            "</figure>",
        ]:
            self.assertTrue(token in words, f"{token} is not in words")

    def test_document_to_html(self):
        for asset in ["amzn_q2.png", "fake_id.png", "form_1005.png", "form.png", "in-table-title.png", "matrix.png", "patient_intake_form_sample.png", "paystub_header.png", "paystub_single_table.png", "paystub_tables.png", "reading_order.pdf", "receipt.jpg", "sample-invoice.pdf", "screenshot.png", "single-page-1.png", "single-page-2.png", "test.png", "textractor-singlepage-doc.pdf"]:
            # Testing that no asset causes the output to contain duplicate words
            if os.environ.get("CALL_TEXTRACT"):
                document = self.extractor.analyze_document(
                    os.path.join(self.fixture_directory, asset),
                    features=[
                        TextractFeatures.LAYOUT,
                        TextractFeatures.TABLES,
                        TextractFeatures.FORMS,
                        TextractFeatures.SIGNATURES
                    ]
                )
                with open(get_fixture_path()[:-5] + "_" + asset + ".json", "w") as f:
                    json.dump(document.response, f)
            else:
                document = Document.open(get_fixture_path()[:-5] + "_" + asset + ".json")

            
            html_document = document.to_html(HTMLLinearizationConfig(
                figure_layout_prefix="<div><p>",
                figure_layout_suffix="</p></div>",
                footer_layout_prefix="<div><p>",
                footer_layout_suffix="</p></div>",
                key_value_layout_prefix="<div><p>",
                key_value_layout_suffix="</p></div>",
                page_num_prefix="<div><p>",
                page_num_suffix="</p></div>",
            ))
            root = html.fromstring(html_document)
            
            for node in root.getiterator():
                if node.text and node.tag not in ["p", "h1", "h2", "h3", "h4", "h5", "th", "td", "caption"]:
                    raise Exception(f"Tag {node.tag} contains text {node.text}")
                
    def test_document_to_markdown(self):
        for asset in ["amzn_q2.png", "fake_id.png", "form_1005.png", "form.png", "in-table-title.png", "matrix.png", "patient_intake_form_sample.png", "paystub_header.png", "paystub_single_table.png", "paystub_tables.png", "reading_order.pdf", "receipt.jpg", "sample-invoice.pdf", "screenshot.png", "single-page-1.png", "single-page-2.png", "test.png", "textractor-singlepage-doc.pdf"]:
            # Testing that no asset causes the output to contain duplicate words
            if os.environ.get("CALL_TEXTRACT"):
                document = self.extractor.analyze_document(
                    os.path.join(self.fixture_directory, asset),
                    features=[
                        TextractFeatures.LAYOUT,
                        TextractFeatures.TABLES,
                        TextractFeatures.FORMS,
                        TextractFeatures.SIGNATURES
                    ]
                )
                with open(get_fixture_path()[:-5] + "_" + asset + ".json", "w") as f:
                    json.dump(document.response, f)
            else:
                document = Document.open(get_fixture_path()[:-5] + "_" + asset + ".json")

            document.to_markdown()

if __name__ == "__main__":
    TestGetTextAndWords().test_document_to_html()


import os
import unittest
from tests.utils import get_fixture_path
from textractor import Textractor
from textractor.entities.document import Document
from textractor.exceptions import InvalidProfileNameError
from textractor.data.constants import TextractFeatures

from .utils import save_document_to_fixture_path

class TestSignature(unittest.TestCase):
    def test_signature(self):
        # Insert credentials here to run test
        profile_name = "default"
        current_directory = os.path.abspath(os.path.dirname(__file__))

        if profile_name is None:
            raise InvalidProfileNameError(
                "Textractor could not be initialized. Populate profile_name with a valid input in tests/test_table.py."
            )

        if os.environ.get("CALL_TEXTRACT"):
            extractor = Textractor(
                profile_name=profile_name, kms_key_id=""
            )
            document = extractor.analyze_document(
                file_source=os.path.join(current_directory, "fixtures/signature.jpg"),
                features=[TextractFeatures.SIGNATURES],
                save_image=False,
            )
            save_document_to_fixture_path(document)
        else:
            document = Document.open(get_fixture_path())

        self.assertEqual(len(document.signatures), 1)
        self.assertEqual(len(document.pages[0].signatures), 1)



import unittest

from textractor.data.constants import SelectionStatus, SELECTED
from textractor.entities.bbox import BoundingBox
from textractor.entities.selection_element import SelectionElement


class TestSelectionElement(unittest.TestCase):
    def setUp(self):
        self.checkbox_bb = {
            "Width": 0.09679746627807617,
            "Height": 0.008036591112613678,
            "Left": 0.08719838410615921,
            "Top": 0.5354593992233276,
        }


    def test_is_selected(self):
        """Test case to return the selection status of the checkbox"""
        checkbox = SelectionElement(
            entity_id="checkbox-id",
            bbox=BoundingBox.from_normalized_dict(self.checkbox_bb, spatial_object=None),
            status=SelectionStatus.SELECTED,
            confidence=100,
        )
        self.assertTrue(checkbox.is_selected())


    def test_words(self):
        """Test case to return the words of the checkbox"""
        checkbox = SelectionElement(
            entity_id="checkbox-id",
            bbox=BoundingBox.from_normalized_dict(self.checkbox_bb, spatial_object=None),
            status=SelectionStatus.SELECTED,
            confidence=100,
        )
        self.assertEqual(checkbox.words, [])


    def test_repr(self):
        """Test case to return the selection status of the checkbox as string"""
        checkbox = SelectionElement(
            entity_id="checkbox-id",
            bbox=BoundingBox.from_normalized_dict(self.checkbox_bb, spatial_object=None),
            status=SelectionStatus.SELECTED,
            confidence=100,
        )
        self.assertEqual(checkbox.__repr__(), "[X]")


    def test_set_page(self):
        """Test case setter for the page attribute"""
        checkbox = SelectionElement(
            entity_id="checkbox-id",
            bbox=BoundingBox.from_normalized_dict(self.checkbox_bb, spatial_object=None),
            status=SelectionStatus.SELECTED,
            confidence=100,
        )
        checkbox.page = 2
        self.assertEqual(checkbox.page, 2)


    def test_set_page_id(self):
        """Test case setter for the page_id attribute"""
        checkbox = SelectionElement(
            entity_id="checkbox-id",
            bbox=BoundingBox.from_normalized_dict(self.checkbox_bb, spatial_object=None),
            status=SelectionStatus.SELECTED,
            confidence=100,
        )
        checkbox.page_id = "page-id"
        self.assertEqual(checkbox.page_id, "page-id")



import html
from textractor.data.html_linearization_config import HTMLLinearizationConfig

def add_id_to_html_tag(prefix, id, config):
    if not isinstance(config, HTMLLinearizationConfig) or not prefix:
        return prefix
    if config.add_ids_to_html_tags:
        return prefix[:-1] + f' id="{id}"' + prefix[-1]
    elif config.add_short_ids_to_html_tags:
        return prefix[:-1] + f' id="{id[:8]}"' + prefix[-1]
    else:
        return prefix

def escape_text(text, config):
    if not isinstance(config, HTMLLinearizationConfig):
        return text
    else:
        return html.escape(text)



import os
import PIL
import unittest
import boto3
import uuid
import logging
import subprocess
from typing import List
from tests.utils import get_fixture_path

from textractor import Textractor
from textractor.entities.document import Document
from textractor.entities.lazy_document import LazyDocument
from textractor.data.constants import TextractFeatures
from textractor.exceptions import InvalidProfileNameError, S3FilePathMissing
from textractor.utils.s3_utils import upload_to_s3, delete_from_s3


def run_command(cmds: List):
    output = subprocess.run(cmds, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    if output.returncode != 0:
        logging.error(output.stdout)
        logging.error(output.stderr)
        raise Exception(output.stderr)

class TestTextractor(unittest.TestCase):
    def setUp(self):
        # insert credentials and filepaths here to run test
        #self.profile_name = "default"
        self.bucket_name = os.environ.get("S3_BUCKET", "textractor-tests")
        if os.environ.get("CALL_TEXTRACT"):
            self.s3_client = boto3.session.Session(
                #profile_name=self.profile_name
            ).client("s3", region_name="us-west-2")

            self.current_directory = os.path.abspath(os.path.dirname(__file__))
            for asset in ["single-page-1.png", "textractor-multipage-doc.pdf"]:
                upload_to_s3(self.s3_client, f"s3://{self.bucket_name}/{asset}", os.path.join(self.current_directory, f"fixtures/{asset}"))

            self.s3_image_file = f"s3://{self.bucket_name}/single-page-1.png"
            self.s3_multipage_pdf_file = f"s3://{self.bucket_name}/textractor-multipage-doc.pdf"
            self.s3_output_path = f"s3://{self.bucket_name}/output"
            self.s3_upload_path = f"s3://{self.bucket_name}/upload"
            self.image_1 = PIL.Image.open(os.path.join(self.current_directory, "fixtures/single-page-1.png"))
            self.image_2 = PIL.Image.open(os.path.join(self.current_directory, "fixtures/single-page-2.png"))

    @unittest.skipIf(not os.environ.get("CALL_TEXTRACT"), "CLI tests only work with CALL_TEXTRACT enabled")
    def test_detect_document_text(self):
        run_command([
            "textractor",
            "detect-document-text",
            os.path.join(self.current_directory, "fixtures/single-page-1.png"),
            "output.json",
        ])

    @unittest.skipIf(not os.environ.get("CALL_TEXTRACT"), "CLI tests only work with CALL_TEXTRACT enabled")
    def test_detect_document_text_single_page_pdf_input(self):
        run_command([
            "textractor",
            "detect-document-text",
            os.path.join(self.current_directory, "fixtures/textractor-singlepage-doc.pdf"),
            "output.json",
        ])

    @unittest.skipIf(not os.environ.get("CALL_TEXTRACT"), "CLI tests only work with CALL_TEXTRACT enabled")
    def test_textractor_s3_image_input(self):
        run_command([
            "textractor",
            "detect-document-text",
            self.s3_image_file,
            "output.json",
        ])

    @unittest.skipIf(not os.environ.get("CALL_TEXTRACT"), "CLI tests only work with CALL_TEXTRACT enabled")
    def test_textractor_start_document_text_detection(self):
        run_command([
            "textractor",
            "start-document-text-detection",
            os.path.join(self.current_directory, "fixtures/textractor-multipage-doc.pdf"),
            "--s3-upload-path",
            self.s3_upload_path,
            "--s3-output-path",
            self.s3_output_path,
        ])

    @unittest.skipIf(not os.environ.get("CALL_TEXTRACT"), "CLI tests only work with CALL_TEXTRACT enabled")
    def test_textractor_analyze_document(self):
        run_command([
            "textractor",
            "analyze-document",
            os.path.join(self.current_directory, "fixtures/single-page-1.png"),
            "output.json",
            "--features",
            "TABLES",
            "FORMS",
        ])

    @unittest.skipIf(not os.environ.get("CALL_TEXTRACT"), "CLI tests only work with CALL_TEXTRACT enabled")
    def test_textractor_analyze_document_multipage_pdf(self):
        run_command([
            "textractor",
            "start-document-analysis",
            os.path.join(self.current_directory, "fixtures/textractor-multipage-doc.pdf"),
            "--s3-upload-path",
            self.s3_upload_path,
            "--s3-output-path",
            self.s3_output_path,
            "--features",
            "TABLES",
            "FORMS",
        ])

    @unittest.skipIf(not os.environ.get("CALL_TEXTRACT"), "CLI tests only work with CALL_TEXTRACT enabled")
    def test_textractor_start_document_analysis_multipage_pdf_s3(self):
        run_command([
            "textractor",
            "start-document-analysis",
            self.s3_multipage_pdf_file,
            "--s3-upload-path",
            self.s3_upload_path,
            "--s3-output-path",
            self.s3_output_path,
            "--features",
            "TABLES",
            "FORMS",
        ])



"""All test cases for methods in KeyValue class"""

from typing import List
import unittest

from textractor.entities.key_value import KeyValue
from textractor.entities.value import Value
from textractor.entities.word import Word
from textractor.entities.line import Line
from textractor.entities.bbox import BoundingBox
from textractor.data.constants import TextTypes
from textractor.visualizers.entitylist import EntityList

class TestKeyValue(unittest.TestCase):
    def setUp(self):
        # dummy values for bounding boxes
        self.key_bb = {
            "Width": 0.09679746627807617,
            "Height": 0.008036591112613678,
            "Left": 0.08719838410615921,
            "Top": 0.5354593992233276,
        }
        self.value_bb = {
            "Width": 0.02524515800178051,
            "Height": 0.01746263913810253,
            "Left": 0.18779051303863525,
            "Top": 0.4229613244533539,
        }
        self.word_bb_1 = {
            "Width": 0.10809839516878128,
            "Height": 0.01363567914813757,
            "Left": 0.036161474883556366,
            "Top": 0.03439946100115776,
        }
        self.word_bb_2 = {
            "Width": 0.18033172190189362,
            "Height": 0.01742148958146572,
            "Left": 0.22032427787780762,
            "Top": 0.03645794093608856,
        }
        self.word_bb_3 = {
            "Width": 0.03744738921523094,
            "Height": 0.016524378210306168,
            "Left": 0.4087739884853363,
            "Top": 0.0368686243891716,
        }
        self.word_1 = Word(
            entity_id="word-id-1",
            bbox=BoundingBox.from_normalized_dict(self.word_bb_1, spatial_object=None),
            text="TEST",
            text_type=TextTypes.PRINTED,
        )
        self.word_2 = Word(
            entity_id="word-id-2",
            bbox=BoundingBox.from_normalized_dict(self.word_bb_2, spatial_object=None),
            text="KEY",
            text_type=TextTypes.HANDWRITING,
        )
        self.word_3 = Word(
            entity_id="word-id-3",
            bbox=BoundingBox.from_normalized_dict(self.word_bb_3, spatial_object=None),
            text="WORDS",
            text_type=TextTypes.PRINTED,
        )
        self.word_objs = [self.word_1, self.word_2, self.word_3]

        self.value = Value(
            entity_id="value-id",
            bbox=BoundingBox.from_normalized_dict(self.value_bb, spatial_object=None),
        )
        self.value_word = Word(
            entity_id="value-word-id",
            bbox=BoundingBox.from_normalized_dict(self.word_bb_1, spatial_object=None),
            text="TEST VALUE",
            text_type=TextTypes.HANDWRITING,
        )
        self.value.words = [self.value_word]

        self.kv = KeyValue(
            entity_id="kv-id",
            bbox=BoundingBox.from_normalized_dict(self.key_bb, spatial_object=None),
            contains_checkbox=False,
            value=None,
        )
        self.kv.page_id = "page-id"
        self.kv.key = self.word_objs
        self.kv.page = 2
        self.kv.value = self.value

    def test_key(self):
        """Test case to set Key field to a key-value pair"""
        self.assertIsInstance(self.kv.key, List)


    def test_value(self):
        """Test case to set Value field to a key-value pair"""
        self.assertEqual(self.kv.value, self.value)


    def test_words(self):
        """Test case to get words of a key-value pair"""
        self.assertEqual(set(self.kv.words), set(EntityList(self.word_objs + [self.value_word])))


    def test_set_page(self):
        """Test case setter for the page attribute"""
        self.assertEqual(self.kv.page, 2)


    def test_set_page_id(self):
        """Test case setter for the page_id attribute"""
        self.assertEqual(self.kv.page_id, "page-id")


    def test_get_words_by_type(self):
        """Test case to retrieve words of specified text_type in key-value pair"""
        # asserting on set because ordering in returned word lists isn't important
        self.assertEqual(
            set(self.kv.get_words_by_type(text_type=TextTypes.PRINTED)),
            set(EntityList([self.word_1, self.word_3])
        ))
        self.assertEqual(
            set(self.kv.get_words_by_type(text_type=TextTypes.HANDWRITING)),
            set(EntityList([self.word_2, self.value_word])
        ))


    def test_repr(self):
        """Test case to retrieve KeyValue representation"""
        self.assertEqual(self.kv.__repr__(), "TEST KEY WORDS : TEST VALUE")



import json
import os
import PIL
import unittest
from textractor import Textractor
from textractor.entities.document import Document
from textractor.data.constants import TextractFeatures
from textractor.exceptions import InvalidProfileNameError, NoImageException, S3FilePathMissing

from .utils import get_fixture_path

class TestTextractorAnalyzeExpense(unittest.TestCase):
    def setUp(self):
        # insert credentials and filepaths here to run test
        self.profile_name = "default"
        self.current_directory = os.path.abspath(os.path.dirname(__file__))
        self.image_path = os.path.join(self.current_directory, "fixtures/receipt.jpg")
        self.image = PIL.Image.open(self.image_path)

        if self.profile_name is None:
            raise InvalidProfileNameError(
                "Textractor could not be initialized. Populate profile_name with a valid input in tests/test_textractor.py."
            )
        if os.environ.get("CALL_TEXTRACT"):
            self.extractor = Textractor(
                profile_name=self.profile_name, kms_key_id=""
            )

    def test_analyze_expense_from_path(self):
        # Testing local single image input
        if os.environ.get("CALL_TEXTRACT"):
            document = self.extractor.analyze_expense(file_source=self.image_path)
            with open(get_fixture_path(), "w") as f:
                json.dump(document.response, f)
        else:
            document = Document.open(get_fixture_path())

        self.assertIsInstance(document, Document)
        self.assertEqual(len(document.pages), 1)
        self.assertEqual(document.expense_documents[0].summary_fields.TOTAL[0].value.text, "$1810.46")
        self.assertEqual(len(document.expense_documents[0].summary_groups.VENDOR), 2)
        self.assertEqual(len(document.expense_documents[0].line_items_groups[0].to_pandas()), 4,
                         "There are 4 line item in the receipts")

    def test_analyze_expense_from_image(self):
        # Testing local single image input
        if os.environ.get("CALL_TEXTRACT"):
            document = self.extractor.analyze_expense(file_source=self.image)
            with open(get_fixture_path(), "w") as f:
                json.dump(document.response, f)
        else:
            document = Document.open(get_fixture_path())

        self.assertIsInstance(document, Document)
        self.assertEqual(len(document.pages), 1)
        self.assertEqual(document.expense_documents[0].summary_fields.TOTAL[0].value.text, "$1810.46")
        self.assertEqual(len(document.expense_documents[0].summary_groups.VENDOR), 2)
        self.assertEqual(len(document.expense_documents[0].line_items_groups[0].to_pandas()), 4,
                         "There are 4 line item in the receipts")


class TestTextractorAnalyzeExpenseNoSummary(unittest.TestCase):
    def setUp(self):
        # insert credentials and filepaths here to run test
        self.profile_name = "default"
        self.current_directory = os.path.abspath(os.path.dirname(__file__))
        self.image_path = os.path.join(self.current_directory, "fixtures/receipt_no_summary.png")

        if self.profile_name is None:
            raise InvalidProfileNameError(
                "Textractor could not be initialized. Populate profile_name with a valid input in tests/test_textractor.py."
            )
        if os.environ.get("CALL_TEXTRACT"):
            self.extractor = Textractor(
                profile_name=self.profile_name, kms_key_id=""
            )

    def test_analyze_expense_no_summary_fields(self):
        """Correctly load expense line items where no summary fields were recognized

        Per: https://github.com/aws-samples/amazon-textract-textractor/issues/370
        """
        if os.environ.get("CALL_TEXTRACT"):
            document = self.extractor.analyze_expense(file_source=self.image_path)
            with open(get_fixture_path(), "w") as f:
                json.dump(document.response, f)
        else:
            document = Document.open(get_fixture_path())

        self.assertIsInstance(document, Document)
        self.assertEqual(len(document.expense_documents), 1)
        self.assertGreater(len(document.expense_documents[0].line_items_groups), 0)

if __name__ == "__main__":
    test = TestTextractorAnalyzeExpense()
    test.setUp()
    test.test_analyze_expense_from_path()


import json
import os
import random
import PIL
import unittest
import boto3
import uuid
import logging
from tests.utils import get_fixture_path

from textractor import Textractor
from textractor.data.constants import TextractFeatures
from textractor.entities.document import Document
from textractor.exceptions import InvalidProfileNameError, S3FilePathMissing
from textractor.utils.s3_utils import upload_to_s3, delete_from_s3

class TestParseNoFail(unittest.TestCase):
    """The tests below are fuzzing tests and are disabled in the CI test suite.
    They are meant to generate random permutations of the input JSON response and
    ensure that the parser does not raise any exception. Their results may be flaky
    due to the randomness

    :param unittest: _description_
    :type unittest: _type_
    """
    def setUp(self):
        # insert credentials and filepaths here to run test
        self.profile_name = "default"
        self.current_directory = os.path.abspath(os.path.dirname(__file__))
        self.saved_api_responses_directory = os.path.join(self.current_directory, "fixtures", "saved_api_responses")
        self.deletion_rate = 0.5

    def test_parse_no_fail(self):
        for asset in os.listdir(self.saved_api_responses_directory):
            # Testing that no asset causes the output to contain duplicate words
            with open(os.path.join(self.saved_api_responses_directory, asset)) as f:
                response = json.load(f)

            if not "Blocks" in response:
                continue

            index_to_remove = []
            for i in range(len(response["Blocks"])):
                if response["Blocks"][i]["BlockType"] != "PAGE" and random.random() <= self.deletion_rate:
                    index_to_remove.append(i)

            for i in sorted(index_to_remove, reverse=True):
                response["Blocks"].pop(i)

            document = Document.open(response)
            document.get_text_and_words()



import logging
from textractor.data.constants import (
    LAYOUT_FIGURE,
    LAYOUT_LIST,
    LAYOUT_TABLE,
    LAYOUT_KEY_VALUE,
    LAYOUT_TEXT,
    LAYOUT_TITLE,
    LAYOUT_HEADER,
    LAYOUT_FOOTER,
    LAYOUT_SECTION_HEADER,
    LAYOUT_PAGE_NUMBER,
)

logger = logging.getLogger(__name__)

def converter(response):
    blocks_to_delete = []
    page_blocks = []
    try:
        for i, block in enumerate(response["Blocks"]):
            if block.get("BlockType") == "PAGE":
                page_blocks.append(block)
            elif block.get("BlockType", "").startswith("LAYOUT_FIGURE_"):
                block["BlockType"] = LAYOUT_TEXT
            elif (
                block.get("BlockType", "").startswith("LAYOUT_") and
                block.get("BlockType") not in [
                    LAYOUT_TEXT,
                    LAYOUT_TITLE,
                    LAYOUT_HEADER,
                    LAYOUT_FOOTER,
                    LAYOUT_SECTION_HEADER,
                    LAYOUT_PAGE_NUMBER,
                    LAYOUT_LIST,
                    LAYOUT_FIGURE,
                    LAYOUT_TABLE,
                    LAYOUT_KEY_VALUE,
                ]
            ):
                block["BlockType"] = LAYOUT_FIGURE
            elif block.get("BlockType") == LAYOUT_FIGURE and "CONTAINER" in block.get("EntityTypes", []):
                blocks_to_delete.append((i, block))
        
        blocks_to_delete_id_set = set([b["Id"] for _, b in blocks_to_delete])
        for page_block in page_blocks:
            for relationship in page_block.get("Relationships", []):
                if relationship["Type"] == "CHILD":
                    relationship["Ids"] = [
                        id
                        for id in relationship["Ids"]
                        if id not in blocks_to_delete_id_set
                    ]
                    break
            
        for i, block in blocks_to_delete[::-1]:
            del response["Blocks"][i]
    except Exception as ex:
        logger.warning(f"Failed to convert the response for backward compatibility. {str(ex)}")
    
    return response



import os
import PIL
import unittest
import boto3
import uuid
from tests.utils import get_fixture_path

from textractor import Textractor
from textractor.entities.document import Document
from textractor.entities.lazy_document import LazyDocument
from textractor.data.constants import TextractFeatures
from textractor.exceptions import InvalidProfileNameError, S3FilePathMissing


class TestTextractor(unittest.TestCase):
    def setUp(self):
        # insert credentials and filepaths here to run test
        if os.environ.get("CALL_TEXTRACT"):
            self.profile_name = "default"
            self.current_directory = os.path.abspath(os.path.dirname(__file__))
            self.extractor = Textractor(
                profile_name=self.profile_name, kms_key_id=""
            )

    @unittest.skipIf(not os.environ.get("CALL_TEXTRACT"), "This test only work with CALL_TEXTRACT enabled")
    def test_detect_document_text(self):
        # Testing local single image input
        document = self.extractor.detect_document_text(
            file_source=os.path.join(self.current_directory, "fixtures/single-page-1.png"),
        )

        out1 = document.words.visualize()
        out2 = document.words.visualize(with_text=False)
        out3 = (document.words + document.lines).visualize()

    @unittest.skipIf(not os.environ.get("CALL_TEXTRACT"), "This test only work with CALL_TEXTRACT enabled")
    def test_textractor_analyze_document(self):
        # Testing analyze_document() with local single image input
        document = self.extractor.analyze_document(
            file_source=os.path.join(self.current_directory, "fixtures/amzn_q2.png"),
            features=[TextractFeatures.TABLES, TextractFeatures.FORMS],
        )

        out1 = document.tables.visualize()
        out2 = document.tables[0].visualize(with_text=False)
        out3 = document.pages[0].visualize()


import boto3
import os
import PIL
import unittest
import json
from tests.utils import get_fixture_path

from textractor import Textractor
from textractor.entities.document import Document
from textractor.entities.lazy_document import LazyDocument
from textractor.visualizers.entitylist import EntityList
from textractor.data.constants import TextractFeatures
from textractor.exceptions import InvalidProfileNameError, S3FilePathMissing
from textractor.utils.s3_utils import upload_to_s3, delete_from_s3

class TestWordOrdering(unittest.TestCase):
    def setUp(self):
        # insert credentials and filepaths here to run test
        self.profile_name = "default"
        self.bucket_name = os.environ.get("S3_BUCKET", "textractor-tests")
        if os.environ.get("CALL_TEXTRACT"):
            self.s3_client = boto3.session.Session(
                profile_name=self.profile_name
            ).client("s3", region_name="us-west-2")

            if self.profile_name is None:
                raise InvalidProfileNameError(
                    "Textractor could not be initialized. Populate profile_name with a valid input in tests/test_textractor.py."
                )
            self.current_directory = os.path.abspath(os.path.dirname(__file__))
            self.extractor = Textractor(
                profile_name=self.profile_name, kms_key_id=""
            )

    def test_word_ordering_in_cell(self):
        if os.environ.get("CALL_TEXTRACT"):
            document = self.extractor.analyze_document(
                file_source=os.path.join(self.current_directory, "fixtures/reading_order.pdf"),
                features=[TextractFeatures.TABLES]
            )
            with open(get_fixture_path(), "w") as fh:
                json.dump(document.response, fh)
        else:
            document = Document.open(get_fixture_path())

        self.assertEqual(document.tables[0].table_cells[0].text.strip(), "Are those Words in order?")

